% This m file evaluates the Sine and Cosine outputs stored in
% SOUT_results.txt and COUT_results.txt.
% 2^20 samples are required for each output. 

close all
clear
clc

%==============================================================%
% Set the design variables m and d.

m=30; % number of Accumulator bits
d=10; % number of DAC bits

fclk=100e6; % clock frequency

%==============================================================%

nos=2^20; %number of samples

fid=fopen('SOUT_results.txt','r');
SOUT=transpose(fscanf(fid,'%d\n'));
fclose(fid);

fid=fopen('COUT_results.txt','r');
COUT=transpose(fscanf(fid,'%d\n'));
fclose(fid);



sinc=sin(pi*(0:nos/2-1)/nos)./(pi*(0:nos/2-1)/nos);
sinc(1)=0.5;

SpectrumCos=1/nos*fft(COUT/2^d);
SpectrumSin=1/nos*fft(SOUT/2^d);
SpectrumCos=SpectrumCos(1:nos/2).*sinc*2;
SpectrumSin=SpectrumSin(1:nos/2).*sinc*2;
Freqs=fclk/nos*(0:nos/2-1);

figure
plot(Freqs*1e-6,20*log10(abs(SpectrumSin+1e-12)))
xlabel('freq (MHz)');
ylabel('Amplitude (dB)')
title('Sine Spectrum');

figure
plot(Freqs*1e-6,20*log10(abs(SpectrumCos+1e-12)))
xlabel('freq (MHz)');
ylabel('Amplitude (dB)')
title('Cosine Spectrum');

CosFundIndex=find(SpectrumCos==max(SpectrumCos(2:end)));
CosFund=SpectrumCos(CosFundIndex);
CosSpur=sort(abs(SpectrumCos));

SinFundIndex=find(SpectrumSin==max(SpectrumSin(2:end)));
SinFund=SpectrumSin(SinFundIndex);
SinSpur=sort(abs(SpectrumSin));

display('Cosine Information:')
display(['DC: ' num2str(abs(SpectrumCos(1))) ' | FundFreq: ' ...
    num2str(Freqs(CosFundIndex)/1e6) ' MHz | FundAmplitude: ' ...
    num2str(abs(CosFund)) ' (' ... 
    num2str(20*log10(abs(CosFund))) ' dB)']);

display(['Max Spur: ' num2str(20*log10(CosSpur(end-2)/CosSpur(end-1))) ' dB below fundumental']);
display('----------------------------------------------------------')

display('Sine Information:')
display(['DC: ' num2str(abs(SpectrumSin(1))) ' | FundFreq: ' ...
    num2str(Freqs(SinFundIndex)/1e6) ' MHz | FundAmplitude: ' ...
    num2str(abs(SinFund)) ' (' ... 
    num2str(20*log10(abs(SinFund))) ' dB)']);

display(['Max Spur: ' num2str(20*log10(SinSpur(end-2)/SinSpur(end-1))) ' dB below fundumental']);
display('----------------------------------------------------------')

display(['Sine and Cosine phase difference: ' num2str((phase(CosFund)-phase(SinFund))*180/pi)])

figure
hold on
plot((0:1/fclk:(nos-1)/fclk)*1e3,SOUT,'b')
plot((0:1/fclk:(nos-1)/fclk)*1e3,COUT,'r')
xlabel('time (msec)')
ylabel('Amplitude')
