% This m file generates the Sine and Cosine outputs according 
% to m, p, and d design variables and also the desired FCW.
% In order to reduce the required memory, only 2^20 samples are considered.
% Therefore, all (m-20) LSB bits in FCW should be zero to guarantee
% FFT calculations. 

close all
clear
clc

%==============================================================%
% Set the design variables m, p, and d, and also the target FCW.

m=30; % number of Accumulator bits
p=10; % number of bits after truncation
d=10; % number of DAC bits

fclk=100e6; % clock frequency

FCW=bin2dec('00 0000 0000 0001 0000 0000 0000 0000');

%==============================================================%

nos=2^20; %number of samples

phase_acc=mod(0:FCW:(nos-1)*FCW,2^m);
phase_trunc=floor(phase_acc/2^(m-p));
phi=phase_trunc*pi/2^(p-1);

%normalized inverse sinc: 2/pi*(pi*x)/sin(pi*x), where x=FCW/2^m
isinc=2*FCW/2^m/sin(pi*FCW/2^m);


COUT=round((0.5+0.5*cos(phi)*isinc)*2^d);
SOUT=round((0.5+0.5*sin(phi)*isinc)*2^d);


fid=fopen('SOUT_results.txt','wt');
fprintf(fid,'%d\n',SOUT);
fclose(fid);

fid=fopen('COUT_results.txt','wt');
fprintf(fid,'%d\n',COUT);
fclose(fid);


