--------------------------------------------------------------------------------
--  
-- Project Name:  ASIC design contest
-- Visit http://fpgasoc.sbu.ac.ir
-- Description:   
-- 
-- VHDL Testbench for DDS module.

-- The testbench programs the Frequency Control Word (FCW) using UART. Then, after
-- 10 CLK periods, the "Start" command is produced. The testbench then waits for 10
-- additional CLK periods to store 2^m samples of the Sine and Cosine outputs (DOUT
-- and SOUT) in two files. The constants under "User constant" section should be
-- modified according to the user's design. Also make sure to set a sufficient simulation
-- time to capture all 2^m required samples.


-- 
-- Revision:2
--
--------------------------------------------------------------------------------

LIBRARY ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use STD.textio.all;
use ieee.std_logic_textio.all;
 
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;
 
ENTITY testbench IS
END testbench;
 
ARCHITECTURE behavior OF testbench IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    -- User constant 
   
	constant d				: integer	:= 10;	
	constant CLK_ferq		: integer	:= 100_000_000;
	constant CLK_period 	: time 		:= 10 ns;
	constant fcw			: std_logic_vector (39 downto 0)	:= x"0000100000";	
	
	----------------------------------------
	constant	baud_rate	:	integer	:=115_200;
	
	
    COMPONENT DDS
    PORT(
         RXD : IN  std_logic;
         Start : IN  std_logic;
         CLK : IN  std_logic;
         SOUT : OUT  std_logic_vector(d-1 downto 0);
         COUT : OUT  std_logic_vector(d-1 downto 0)
        );
    END COMPONENT;
	 
	
	
   --Inputs
   signal RXD : std_logic := '1';
   signal Start : std_logic := '0';
   signal CLK : std_logic := '0';
	signal baud : std_logic := '0';

 	--Outputs
   signal SOUT : std_logic_vector(d-1 downto 0);
   signal COUT : std_logic_vector(d-1 downto 0);

	file file_RESULTS,file_RESULTC : text;
  
	


 
BEGIN
	
   uut: DDS PORT MAP (
          RXD => RXD,
          Start => Start,
          CLK => CLK,
          SOUT => SOUT,
          COUT => COUT
        );
		  
		  
	sendfcw: PROCESS
		
		begin
		wait for 100 us;
		if start='0'	then
		for i in 0 to	4	loop
			RXD<='0';
			wait for 8.68 us ;
			for j in 0 to 7	loop
				RXD<= fcw(i*8+j);
				baud<= not baud;
				wait for 8.68 us ;
			end loop;
			RXD<='1';
			wait for 2*8.68 us;
		end loop;
		end if;
		wait for 10*CLK_period;
		start<='1';
		
	end process;
					
			

   -- Clock process definitions
   CLK_process :process
   begin
		CLK <= '0';
		wait for CLK_period/2;
		CLK <= '1';
		wait for CLK_period/2;
   end process;
 

   -- Stimulus process
   stim_proc: process 
	variable v_OLINES,v_OLINEC     : line;
   begin		

		
		file_open(file_RESULTS, "SOUT_results.txt", write_mode);
		file_open(file_RESULTC, "COUT_results.txt", write_mode);
		wait for 577515 ns;
		for i in 0 to 1048575 loop
		wait until rising_edge(clk) ;
		
		if start='1' then
		write(v_OLINES, to_integer(unsigned(SOUT)), right);	
      writeline(file_RESULTS, v_OLINES);
		
		write(v_OLINEC, to_integer(unsigned(COUT)), right);	
      writeline(file_RESULTC, v_OLINEC);
		end if;
		end loop;
      wait;
   end process;

END;
